library(rbenchmark)

random_text <- function() {
    intToUtf8(sample(65:90, sample(20:30, 1, replace = TRUE),
                     replace = TRUE))
}

number_of_words <- 1e5L
number_of_unique_words <- 1e4L

set.seed(1)
text <- sample(sapply(seq_len(number_of_unique_words),
                      function(x) random_text()),
               number_of_words, replace = TRUE)

inverted_index1 <- function(x) {
    inv_index <- list()
    for ( i in seq_along(x) ) {
        inv_index[[x[i]]] <- c(inv_index[[x[i]]], i)
    }
    inv_index
}

benchmark1 <- benchmark( inv_index1 <- inverted_index1(text) )

inverted_index2 <- function(x) {
    y <- aggregate(i ~ x, FUN=c, data=data.frame(i=seq_along(x), x=x))
    setNames(y$i, y$x)
}

benchmark2 <- benchmark( inv_index2 <- inverted_index2(text) )


require(hash)

inverted_index3 <- function(x) {
    inv_index <- hash()
    for ( i in seq_along(x) ) {
        inv_index[[x[i]]] <- c(inv_index[[x[i]]], i)
    }
    inv_index
}

benchmark3 <- benchmark( inv_index3 <- inverted_index3(text) )

benchmark1
benchmark2
benchmark3

